@@ -1,10 +1,7 @@
 #!/usr/bin/python3
 
 #
-# Like what you see? Join us!
-# https://www.univention.com/about-us/careers/vacancies/
-#
-# Copyright 2011-2022 Univention GmbH
+# Copyright 2011-2021 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -47,8 +44,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
-
-"""Set user_xattr option on ext2/ext3/ext4 filesystems, remount if necessary"""
+"""Set user_xattr option on ext3/ext4 filesystems, remount if necessary"""
 
 from __future__ import print_function
 
@@ -84,9 +80,9 @@ def _modify_extfs_option(options=[], activate=True, devices=[]):
 			if fstab_partition and fstab_partition.type in ('ext3', 'ext4'):
 				target_partitions.append(fstab_partition)
 			else:
-				print('Device could not be found: %s' % device)
+				print('Device could not be found: %s' % (device,))
 	else:
-		for fstype in ('ext2', 'ext3', 'ext4'):
+		for fstype in ('ext3', 'ext4'):
 			for fstab_partition in fs.get(fstype, ignore_root=False):
 				target_partitions.append(fstab_partition)
 
@@ -94,7 +90,7 @@ def _modify_extfs_option(options=[], activate=True, devices=[]):
 		if _do_modify_extfs_option(fstab_partition, options, activate):
 			fs.save()
 			if subprocess.call(('mount', '-o', 'remount', fstab_partition.spec)):
-				print('Remounting partition failed: %s' % fstab_partition.spec)
+				print('Remounting partition failed: %s' % (fstab_partition.spec,))
 
 
 if __name__ == '__main__':
